/* GLCD Library fr's Nokia 6100
   copyright 2004 Hagen Reddmann */

#include <avr/io.h>
#include <avr/pgmspace.h>

#ifndef GLCD_H

#define GLCD_H


#define __LCD_S1D15G10__
//#define __LCD_PCF8833__

#if defined (__AVR_ATmega8__)
  #define LCD_PORT                PORTB
  #define LCD_PIN                 PINB
  #define LCD_DDR                 DDRB
  #define LCD_CS                  PB2   // SS
  #define LCD_SDA                 PB3   // MOSI
  #define LCD_RESET               PB4   // MISO
  #define LCD_SCL                 PB5   // SCK
#elif defined (__AVR_ATmega32__)
  #define LCD_PORT                PORTB
  #define LCD_PIN                 PINB
  #define LCD_DDR                 DDRB
  #define LCD_CS                  PB4   // SS
  #define LCD_SDA                 PB5   // MOSI
  #define LCD_RESET               PB6   // MISO
  #define LCD_SCL                 PB7   // SCK
#elif defined (__AVR_ATmega16__)
  #define LCD_PORT                PORTB
  #define LCD_PIN                 PINB
  #define LCD_DDR                 DDRB
  #define LCD_CS                  PB4   // SS
  #define LCD_SDA                 PB5   // MOSI
  #define LCD_RESET               PB6   // MISO
  #define LCD_SCL                 PB7   // SCK


#elif defined (__AVR_ATmega128__)
  #define LCD_PORT                PORTB
  #define LCD_PIN                 PINB
  #define LCD_DDR                 DDRB
  #define LCD_CS                  0     // SS
  #define LCD_SDA                 2     // MOSI
  #define LCD_RESET               3     // MISO
  #define LCD_SCL                 1     // SCK
#else
  #error Please define in glcd.h the hardware pins for your device !
#endif

#define DELAY_ONE_MILLISECOND   4000  // 1 millisecond at 16Mhz

#define SCREEN_WIDTH            132   // Nokia6100, wobei aber nur 1,1-130,130 sichtbar sind. Denoch ben�igen wir vollen
#define SCREEN_HEIGHT           132   // Zugriff auf den RAM wenn wir zB. das Scrolling nutzen wollen
#define SCREEN_LEFT             1
#define SCREEN_TOP              1
#define SCREEN_RIGHT            (SCREEN_WIDTH -2)
#define SCREEN_BOTTOM           (SCREEN_HEIGHT -2)

#define SCREEN_COLOR            WHITE  // sollte ge�dert werden fr andere Hintergrundfarbe
#define COLOR_TABLE_BITS        2      // 2^2 = 4 Farben in Farbtabelle, wichtig fr Fonts
#define FONT_HEADER_SIZE        7      // Header gr�e eines Fonts

#define RGB(r,g,b) ((r & 0xE0) | ((g & 0xE0) >> 3) | (b >> 6))

#define NONE                    RGB(0x00, 0x20, 0x00)  // == tansparent, ein grn Ton weniger !!
#define BLACK                   RGB(0x00, 0x00, 0x00)  // fr 8 Bit Farben R,G,B  3,3,2 Bits
#define WHITE                   RGB(0xFF, 0xFF, 0xFF)
#define RED                     RGB(0xFF, 0x00, 0x00)
#define GREEN                   RGB(0x00, 0xFF, 0x00)
#define BLUE                    RGB(0x00, 0x00, 0xFF)
#define YELLOW                  RGB(0xFF, 0xFF, 0x00)
#define MAGENTA                 RGB(0xFF, 0x00, 0xFF)
#define CYAN                    RGB(0x00, 0xFF, 0xFF)
#define GRAY                    RGB(0x80, 0x80, 0x40)
#define SILVER                  RGB(0xA0, 0xA0, 0x80)
#define GOLD                    RGB(0xA0, 0xA0, 0x40)


#define USE_ISR               // SPI ISR aktivieren um das Chip Select Signal des Displays zurckzusetzen.
#define USE_LINES             // Linien und Frame Routinen aktivieren, vertikale+horizontale Linien sind immer m�lich mit FilllRect
#define USE_ELLIPSES          // Kreise und Ellipsen
#define USE_FONTS             // Schriftausgaben



typedef uint8_t glcdCoord_t;          // Koordinaten
typedef uint16_t glcdCoord2_t;        // intern zur Koordinatenberechnung
typedef uint8_t glcdColor_t;          // Farbe, fr 256 Farbe uint8_t
typedef uint8_t *glcdFontData_t;      // zeiger auf Fontdaten
typedef uint8_t (*glcdFontFunc_t)(const glcdFontData_t index); // Fontdaten-Lese-Callback, somit k�nen die Fontdaten extern gespeichert sein

typedef struct {
    glcdCoord_t X;
    glcdCoord_t Y;
} glcdPoint_t;

typedef struct {
    glcdCoord_t X1;
    glcdCoord_t Y1;
    glcdCoord_t X2;
    glcdCoord_t Y2;
} glcdRect_t;

typedef struct {
    uint8_t AutoLineFeed:1,        // Fontroutine nutzt einen impliziten Zeilenumbruch innerhalb von glcd_Window
            FixedFont:1;           // Fontroutine soll Font mit fester Zeichenbreite darstellen, ansonsten proportional
} glcdFlags_t;



glcdColor_t     glcd_Colors[1 << COLOR_TABLE_BITS]; // Farbtabelle, Colors[0] = Hintergrund = Fllfarbe, Color[1] = Vordergrund = Linienfarbe
uint8_t         glcd_MemCtrl;
glcdFlags_t     glcd_Flags;
glcdRect_t      glcd_Clip;            // Clipping Koordinaten, alles was ausserhalb ist wird nicht dargestellt

#ifdef USE_FONTS
glcdPoint_t     glcd_Cursor;          // aktuelle Position fr Schriften
glcdRect_t      glcd_Window;          // aktuelles Fenster fr Textausgaben
char            glcd_FontFirstChar;   // erstes Zeichen im Font
char            glcd_FontLastChar;    // letztes Zeichen im Font
uint8_t         glcd_FontWidth;       // fixed Zeichenbreite des Fonts
uint8_t         glcd_FontHeight;      // Zeichenh�e
uint8_t         glcd_FontBitsPixel;   // Farbtiefe des Fonts
glcdFontFunc_t  glcd_FontRead;        // User definierte Readcallback fr die Fontdaten
glcdFontData_t  glcd_FontData;        // die eigentlichen Fontdaten beginnen hier, abh�gig von der Readcallback
#endif

// todo: Fllen mit Brush, Linien mit Pen

void glcdDisplayInit(void);                                   // initialize GLCD
void glcdDisplayCommand(const uint8_t cmd);                   // send GLCD 9bit command
void glcdDisplayData(const uint8_t data);                     // send GLCD 9bit data
void glcdDisplaySend(const uint8_t data, const uint8_t cmd);  // transfer to GLCD
uint32_t glcdDisplayRead(uint8_t bitcount);                   // read bitcount bits from GLCD
void glcdWait(uint16_t ms);

void glcdSetOrientation(const uint8_t orientation);           // Display Orientation {0,1,2,3} -> {0, 90, 180, 270}
void glcdSetAddr(glcdCoord_t x1, glcdCoord_t y1, glcdCoord_t x2, glcdCoord_t y2);
void glcdSetPixel(glcdCoord_t x, glcdCoord_t y, const glcdColor_t color);
void glcdFillRect(glcdCoord_t x1, glcdCoord_t y1, glcdCoord_t x2, glcdCoord_t y2, const glcdColor_t color);
void glcdRectangle(glcdCoord_t x1, glcdCoord_t y1, glcdCoord_t x2, glcdCoord_t y2);
#ifdef USE_LINES
void glcdFrame(glcdCoord_t x1, glcdCoord_t y1, glcdCoord_t x2, glcdCoord_t y2);
void glcdLine(glcdCoord_t x1, glcdCoord_t y1, glcdCoord_t x2, glcdCoord_t y2);
#endif
#ifdef USE_ELLIPSES
void glcdCircle(glcdCoord_t x, glcdCoord_t y, glcdCoord_t r);
void glcdEllipse(glcdCoord_t x, glcdCoord_t y, glcdCoord_t a, glcdCoord_t b);
#endif
#ifdef USE_FONTS
// Font routinen, can be used as Icons
void glcdSelectFont(PGM_VOID_P data, glcdFontFunc_t callback);  // select given font and read above font vaiables in
void glcdNewLine(void);                                         // linefeed in glcd_Window bounds
uint8_t glcdDrawChar(char c);                                   // paint single char
uint8_t glcdCharWidth(char c);                                  // compute pixel width of single char
uint8_t glcdFontRead(const glcdFontData_t index);               // possible Font-Read-Callback for reading from Flash
uint16_t glcdPrint_P(const char *addr);                         // paint nullterminated string stored in ROM
uint16_t glcdCharsWidth_P(const char *addr);                    // compute pixel width of string stored in ROM
uint16_t glcdPrint(const char *addr);                           // paint nullterminated string stored in RAM
uint16_t glcdCharsWidth(const char *addr);                      // compute pixel width of string stored in RAM
#endif

// todo:
// void glcdBezier(?????);

#define glcdBkColor glcd_Colors[0]
#define glcdFgColor glcd_Colors[1]
#define glcdFrColor glcd_Colors[2]
#define glcdShColor glcd_Colors[3]
#define glcdSetColor(index, color) {glcd_Colors[index] = color;}
#define glcdSetBkColor(color) {glcdSetColor(0, color);}
#define glcdSetFgColor(color) {glcdSetColor(1, color);}
#define glcdSetFrColor(color) {glcdSetColor(2, color);}
#define glcdSetShColor(color) {glcdSetColor(3, color);}
#define glcdSetColors(fgColor, bkColor) {glcdSetFgColor(fgColor); glcdSetBkColor(bkColor);}
#define glcdPutPixel(color) {glcdDisplayData(color);}
#define glcdClearScreen() {glcdFillRect(0, 0, SCREEN_WIDTH -1, SCREEN_HEIGHT -1, SCREEN_COLOR);}
#define glcdSetRect(r,x1,y1,x2,y2) {r.X1 = x1; r.Y1 = y1; r.X2 = x2; r.Y2 = y2;}
#define glcdSwapCoord(x, y) {glcdCoord_t t = x; x = y; y = t;}
#define glcdCheckCoord(x, y) {if (x > y) {glcdSwapCoord(x, y)}}

#ifdef USE_FONTS
#define glcdMoveTo(x, y) {glcd_Cursor.X = x; glcd_Cursor.Y = y;}
#endif

// Commands

#define NO_OPERATION            0x00
#define SOFT_RESET              0x01
#define BOOSTER_OFF             0x02
#define BOOSTER_ON              0x03
#define DISPLAY_INFO            0x04   // read 25bits where MSB are dummy trash
#define DISPLAY_STATUS          0x09   // read 33bits where MSB are dummy trash
#define SLEEP_IN                0x10
#define SLEEP_OUT               0x11
#define PARTIAL_MODE_ON         0x12
#define NORMAL_MODE_ON          0x13
#define INVERSION_OFF           0x20
#define INVERSION_ON            0x21
#define PIXELS_OFF              0x22
#define PIXELS_ON               0x23
#define SET_CONTRAST            0x25
  // CONTRAST_VALE              0xXX
#define DISPLAY_OFF             0x28
#define DISPLAY_ON              0x29
#define COLOR_SET               0x2D
  // 8 times red palette        0xXX
  // 8 times green palette      0xXX
  // 4 times blue palette       0xXX
#define PARTIAL_AREA            0x30
  // START_Y                    0xXX
  // STOP_Y                     0xXX
#define SCROLL_DEFINITON        0x33  // always vertical
  // TOP_FIXED_LINES            0xXX
  // SCROLL_LINES               0xXX
  // BOTTOM_FIXED_LINES         0xXX
#define SET_SCROLL              0x37  // SEP scroll entry point
  // ROW_Y                      0xxx

#if defined (__LCD_PCF8833__)
#define MEM_CONTROL             0x36
    #define SET_X_ADDR              0x2A  // ATENTION! when X,Y address are changed BOTH must be changed
    // START_X                    0xXX
    // STOP_X                     0xXX
    #define SET_Y_ADDR              0x2B
    // START_Y                    0xXX
    // STOP_Y                     0xXX
    #define MEM_WRITE               0x2C
    // x times pixel data         0xXX
    // MEM_FLAGS                  0xXX
  #define MEM_90                0     // virtual flag not for PCF8833
  #define MEM_RGB               3     // color order
  #define MEM_LAO               4     // line access order
  #define MEM_VW                5     // vertical write
  #define MEM_MX                6     // mirror X
  #define MEM_MY                7     // mirror Y

#endif

#define IDLE_OFF                0x38
#define IDLE_ON                 0x39
#define COLOR_INTERFACE         0x3A
  // pixelformat                0xXX
  #define COLOR_8_BIT           0x02
  #define COLOR_12_BIT          0x03
  #define COLOR_16_BIT          0x05
#define SET_VOP                 0xB0  // irrelevant for Nokia6100
  // 2x                         0xXX
#define BOTTOM_ROW_SWAP_OFF     0xB4
#define BOTTOM_ROW_SWAP_ON      0xB5
#define TOP_ROW_SWAP_OFF        0xB6
#define TOP_ROW_SWAP_ON         0xB7
#define SUPER_FRAME_INV_OFF     0xB8
#define SUPER_FRAME_INV_ON      0xB9
#define DATA_ORDER_MSB          0xBA
#define DATA_ORDER_LSB          0xBB
#define TEMP_READ               0xC8  // read
  // temp                       0xXX


#if defined (__LCD_S1D15G10__)

    #define MEM_CONTROL 0xbc
    #define MEM_90      7
    #define MEM_RGB     3     // color order
    #define MEM_LAO     4     // line access order
    #define MEM_VW      2     // vertical write
    #define MEM_MX      0     // mirror X
    #define MEM_MY      1     // mirror Y

    #define SET_X_ADDR  0x75  
    #define SET_Y_ADDR  0x15
    #define MEM_WRITE   0x5c

    #define DISON       0xaf
    #define DISOFF      0xae
    #define DISNOR      0xa6
    #define DISINV      0xa7
    #define COMSCN      0xbb
    #define DISCTL      0xca
    #define SLPIN       0x95
    #define SLPOUT      0x94
    #define PASET       0x75
    #define CASET       0x15
    #define DATCTL      0xbc
    #define RGBSET8     0xce
    #define RAMWR       0x5c
    #define RAMRD       0x5d
    #define PTLIN       0xa8
    #define PTLOUT      0xa9
    #define RMWIN       0xe0
    #define RMWOUT      0xee
    #define ASCSET      0xaa
    #define SCSTART     0xab
    #define OSCON       0xd1
    #define OSCOFF      0xd2
    #define PWRCTR      0x20
    #define VOLCTR      0x81
    #define VOLUP       0xd6
    #define VOLDOWN     0xd7
    #define TMPGRD      0x82
    #define EPCTIN      0xcd
    #define EPCOUT      0xcc
    #define EPMWR       0xfc
    #define EPMRD       0xfd
    #define EPSRRD1     0x7c
    #define EPSRRD2     0x7d
    #define NOP         0x25
    
    #define VOLUP_MAX   162
    #define VOLCTR_VAL0 0xff
    #define VOLCTR_VAL1 0x03
    #define PWRCTR_VAL0 0x0f

#endif

#endif


