#include <avr/io.h>
//#include <avr/signal.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "uart.h"
#include "fifo.h"

#define BAUDRATE 9600
#define F_CPU    4000000
#define BUFSIZE_IN  0x40
uint8_t inbuf[BUFSIZE_IN];
fifo_t infifo;

#define BUFSIZE_OUT 0x40
uint8_t outbuf[BUFSIZE_OUT];
fifo_t outfifo;

#define UART_BAUD_SELECT(baudRate,xtalCpu) ((xtalCpu)/((baudRate)*16l)-1)


void uart_init(void)
{
    uint8_t sreg = SREG;
    uint16_t ubrr = UART_BAUD_SELECT(BAUDRATE, F_CPU);
    UBRRH = (uint8_t) (ubrr >> 8);
    UBRRL = (uint8_t) (ubrr);
    cli();
    UCSRB = (1 << RXEN) | (1 << TXEN) | (1 << RXCIE);
    UCSRC = (1 << URSEL) | (1 << UCSZ1) | (1 << UCSZ0);
    do {
        uint8_t dummy;
        (void) (dummy = UDR);
    } while (UCSRA & (1 << RXC));
    UCSRA = (1 << RXC) | (1 << TXC);
    fifo_init(&infifo, inbuf, BUFSIZE_IN);
    fifo_init(&outfifo, outbuf, BUFSIZE_OUT);
    SREG = sreg;
}




SIGNAL(SIG_UART_RECV)
{
    _inline_fifo_put(&infifo, UDR);

}

// Ein Zeichen aus der Ausgabe-FIFO lesen und ausgeben
// Ist das Zeichen fertig ausgegeben, wird ein neuer SIG_UART_DATA-IRQ getriggert
// Ist die FIFO leer, deaktiviert die ISR ihren eigenen IRQ.
SIGNAL(SIG_UART_DATA)
{
    if (outfifo.count > 0)
        UDR = _inline_fifo_get(&outfifo);
    else
        UCSRB &= ~(1 << UDRIE);
}
inline int uart_putc(const uint8_t c)
{
    int ret = fifo_put(&outfifo, c);

    UCSRB |= (1 << UDRIE);

    return ret;
}

int uart_getc_nowait()
{
    return fifo_get_nowait(&infifo);
}

uint8_t uart_getc_wait()
{
    return fifo_get_wait(&infifo);
}

inline void uart_puts(const char *s)
{
    do {
        uart_putc(*s);
    }
    while (*s++);
}

inline void uart_puts_P(PGM_P s)
{
    while (1) {
        unsigned char c = pgm_read_byte(s);
        s++;
        if ('\0' == c)
            break;
        uart_putc(c);
    }
}
